@@ -23,7 +23,8 @@
 extern void afsi_log(char *pattern, ...);
 
 int cm_noLocalMountPoints;
-int cm_fakeDirSize;
+char * cm_FakeRootDir = NULL;
+int cm_fakeDirSize = 0;
 int cm_fakeDirCallback=0;
 int cm_fakeGettingCallback=0;
 cm_localMountPoint_t* cm_localMountPoints;
@@ -152,10 +153,6 @@ void cm_InitFreelance() {
 
     lock_InitializeMutex(&cm_Freelance_Lock, "Freelance Lock");
 
-    // make sure we sync the data version to the cached root scache_t                  
-    if (cm_data.rootSCachep && cm_data.rootSCachep->fid.cell == AFS_FAKE_ROOT_CELL_ID) 
-        cm_data.fakeDirVersion = cm_data.rootSCachep->dataVersion;                          
-                                                                                      
     // yj: first we make a call to cm_initLocalMountPoints
     // to read all the local mount points from the registry
     cm_InitLocalMountPoints();
@@ -169,12 +166,12 @@ void cm_InitFreelance() {
     /* Start the registry monitor */
     phandle = thrd_Create(NULL, 65536, (ThreadFunc) cm_FreelanceChangeNotifier,
                           NULL, 0, &lpid, "cm_FreelanceChangeNotifier");
-    osi_assert(phandle != NULL);
+    osi_assertx(phandle != NULL, "cm_FreelanceChangeNotifier thread create failure");
     thrd_CloseHandle(phandle);
 
     phandle = thrd_Create(NULL, 65536, (ThreadFunc) cm_FreelanceSymlinkChangeNotifier,
                           NULL, 0, &lpid, "cm_FreelanceSymlinkChangeNotifier");
-    osi_assert(phandle != NULL);
+    osi_assertx(phandle != NULL, "cm_FreelanceSymlinkChangeNotifier thread create failure");
     thrd_CloseHandle(phandle);
 #endif
 }
@@ -231,8 +228,12 @@ void cm_InitFakeRootDir() {
     }
 
     dirSize = (curPage+1) *  CM_DIR_PAGESIZE;
-    cm_FakeRootDir = malloc(dirSize);
-    cm_fakeDirSize = dirSize;
+    if (cm_fakeDirSize != dirSize) {
+        if (cm_FakeRootDir)
+            free(cm_FakeRootDir);
+        cm_FakeRootDir = malloc(dirSize);
+        cm_fakeDirSize = dirSize;
+    }
 
     // yj: when we get here, we've figured out how much memory we need and 
     // allocated the appropriate space for it. we now prceed to fill
@@ -266,7 +267,6 @@ void cm_InitFakeRootDir() {
     curChunk = 13;
 
     // stick the first 2 entries "." and ".." in
-    fakeEntry.fid.unique = htonl(1);
     fakeEntry.fid.vnode = htonl(1);
     strcpy(fakeEntry.name, ".");
     currentPos = cm_FakeRootDir + curPage * CM_DIR_PAGESIZE + curChunk * CM_DIR_CHUNKSIZE;
@@ -357,10 +357,10 @@ void cm_InitFakeRootDir() {
 
 int cm_FakeRootFid(cm_fid_t *fidp)
 {
-    fidp->cell = AFS_FAKE_ROOT_CELL_ID;            /* root cell */
-    fidp->volume = AFS_FAKE_ROOT_VOL_ID;           /* root.afs ? */
-    fidp->vnode = 0x1;
-    fidp->unique = 0x1;
+    cm_SetFid(fidp, 
+              AFS_FAKE_ROOT_CELL_ID,            /* root cell */
+              AFS_FAKE_ROOT_VOL_ID,            /* root.afs ? */
+              1, 1);
     return 0;
 }
   
@@ -395,27 +395,20 @@ int cm_reInitLocalMountPoints() {
 
     osi_Log0(afsd_logp,"Invalidating local mount point scp...  ");
 
-    aFid.cell = AFS_FAKE_ROOT_CELL_ID;
-    aFid.volume=AFS_FAKE_ROOT_VOL_ID;
-    aFid.unique=0x1;
-    aFid.vnode=0x2;
+    cm_SetFid(&aFid, AFS_FAKE_ROOT_CELL_ID, AFS_FAKE_ROOT_VOL_ID, 1, 1);
 
     lock_ObtainWrite(&cm_scacheLock);
     lock_ObtainMutex(&cm_Freelance_Lock);  /* always scache then freelance lock */
     for (i=0; i<cm_noLocalMountPoints; i++) {
         hash = CM_SCACHE_HASH(&aFid);
         for (scp=cm_data.scacheHashTablep[hash]; scp; scp=scp->nextp) {
-            if (scp->fid.volume == aFid.volume &&
-                 scp->fid.vnode == aFid.vnode &&
-                 scp->fid.unique == aFid.unique 
-                 ) {
-
+            if (scp != cm_data.rootSCachep && cm_FidCmp(&scp->fid, &aFid) == 0) {
                 // mark the scp to be reused
                 cm_HoldSCacheNoLock(scp);
                 lock_ReleaseWrite(&cm_scacheLock);
-                lock_ObtainMutex(&scp->mx);
+                lock_ObtainWrite(&scp->rw);
                 cm_DiscardSCache(scp);
-                lock_ReleaseMutex(&scp->mx);
+                lock_ReleaseWrite(&scp->rw);
                 cm_CallbackNotifyChange(scp);
                 lock_ObtainWrite(&cm_scacheLock);
                 cm_ReleaseSCacheNoLock(scp);
@@ -426,15 +419,15 @@ int cm_reInitLocalMountPoints() {
                      lscpp = &tscp->nextp, tscp = tscp->nextp) {
                     if (tscp == scp) {
                         *lscpp = scp->nextp;
-			lock_ObtainMutex(&scp->mx);
+			lock_ObtainWrite(&scp->rw);
                         scp->flags &= ~CM_SCACHEFLAG_INHASH;
-			lock_ReleaseMutex(&scp->mx);
+			lock_ReleaseWrite(&scp->rw);
                         break;
                     }
                 }
             }
         }
-        aFid.vnode = aFid.vnode + 1;
+        cm_SetFid(&aFid, AFS_FAKE_ROOT_CELL_ID, AFS_FAKE_ROOT_VOL_ID, aFid.vnode + 1, 1);
     }
     lock_ReleaseWrite(&cm_scacheLock);
     osi_Log0(afsd_logp,"\tall old scp cleared!");
@@ -450,11 +443,6 @@ int cm_reInitLocalMountPoints() {
     cm_InitLocalMountPoints();
     osi_Log0(afsd_logp,"\tcreated new set of localmountpoints!");
 
-    // now we have to free the memory allocated in cm_initfakerootdir
-    osi_Log0(afsd_logp,"Removing old fakedir...  ");
-    free(cm_FakeRootDir);
-    osi_Log0(afsd_logp,"\t\told fakedir removed!");
-
     // then we re-create that dir
     osi_Log0(afsd_logp,"Creating new fakedir...  ");
     cm_InitFakeRootDir();
@@ -477,7 +465,7 @@ long cm_InitLocalMountPoints() {
     char line[512];
     char*t, *t2;
     cm_localMountPoint_t* aLocalMountPoint;
-    char hdir[120];
+    char hdir[260];
     long code;
     char rootCellName[256];
 #if !defined(DJGPP)
@@ -686,7 +674,7 @@ long cm_InitLocalMountPoints() {
     /* What follows is the old code to read freelance mount points 
      * out of a text file modified to copy the data into the registry
      */
-    cm_GetConfigDir(hdir);
+    cm_GetConfigDir(hdir, sizeof(hdir));
     strcat(hdir, AFS_FREELANCE_INI);
     // open the ini file for reading
     fp = fopen(hdir, "r");
@@ -821,7 +809,7 @@ int cm_getNoLocalMountPoints() {
 }
 
 #if !defined(DJGPP)
-long cm_FreelanceMountPointExists(char * filename)
+long cm_FreelanceMountPointExists(char * filename, int prefix_ok)
 {
     char* cp;
     char line[512];
@@ -889,6 +877,11 @@ long cm_FreelanceMountPointExists(char * filename)
                 found = 1;
                 break;
             }
+
+            if (prefix_ok && strlen(shortname) - strlen(filename) == 1 && !strncmp(shortname, filename, strlen(filename))) {
+                found = 1;
+                break;
+            }
         }
         RegCloseKey(hkFreelance);
     }
@@ -898,7 +891,7 @@ long cm_FreelanceMountPointExists(char * filename)
     return found;
 }
 
-long cm_FreelanceSymlinkExists(char * filename)
+long cm_FreelanceSymlinkExists(char * filename, int prefix_ok)
 {
     char* cp;
     char line[512];
@@ -946,6 +939,11 @@ long cm_FreelanceSymlinkExists(char * filename)
                 found = 1;
                 break;
             }
+
+            if (prefix_ok && strlen(shortname) - strlen(filename) == 1 && !strncmp(shortname, filename, strlen(filename))) {
+                found = 1;
+                break;
+            }
         }
         for ( dwIndex = 0; dwIndex < dwSymlinks; dwIndex++ ) {
             TCHAR szValueName[16];
@@ -975,7 +973,7 @@ long cm_FreelanceSymlinkExists(char * filename)
 long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw, cm_fid_t *fidp)
 {
     FILE *fp;
-    char hfile[120];
+    char hfile[260];
     char line[512];
     char fullname[200];
     int n;
@@ -1009,8 +1007,8 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
     }
 
 #if !defined(DJGPP)
-    if ( cm_FreelanceMountPointExists(filename) ||
-         cm_FreelanceSymlinkExists(filename) )
+    if ( cm_FreelanceMountPointExists(filename, 0) ||
+         cm_FreelanceSymlinkExists(filename, 0) )
         return -1;
 #endif
     
@@ -1076,7 +1074,7 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
     } else 
 #endif
     {
-        cm_GetConfigDir(hfile);
+        cm_GetConfigDir(hfile, sizeof(hfile));
         strcat(hfile, AFS_FREELANCE_INI);
         fp = fopen(hfile, "r+");
         if (!fp)
@@ -1096,10 +1094,8 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
     /* cm_reInitLocalMountPoints(); */
-    if (fidp) {
-        fidp->unique = 1;
-        fidp->vnode = cm_noLocalMountPoints + 1;   /* vnode value of last mt pt */
-    }
+    if (fidp)
+        cm_SetFid(fidp, fidp->cell, fidp->volume, ++cm_noLocalMountPoints, 1);
     cm_noteLocalMountPointChange();
     return 0;
 }
@@ -1110,7 +1106,7 @@ long cm_FreelanceRemoveMount(char *toremove)
     char* cp;
     char line[512];
     char shortname[200];
-    char hfile[120], hfile2[120];
+    char hfile[260], hfile2[260];
     FILE *fp1, *fp2;
     int found=0;
 #if !defined(DJGPP)
@@ -1158,6 +1154,7 @@ long cm_FreelanceRemoveMount(char *toremove)
 
             if (!strcmp(shortname, toremove)) {
                 RegDeleteValue( hkFreelance, szValueName );
+                found = 1;
                 break;
             }
         }
@@ -1165,7 +1162,7 @@ long cm_FreelanceRemoveMount(char *toremove)
     } else 
 #endif
     {
-        cm_GetConfigDir(hfile);
+        cm_GetConfigDir(hfile, sizeof(hfile));
         strcat(hfile, AFS_FREELANCE_INI);
         strcpy(hfile2, hfile);
         strcat(hfile2, "2");
@@ -1200,16 +1197,18 @@ long cm_FreelanceRemoveMount(char *toremove)
 
         fclose(fp1);
         fclose(fp2);
-        if (!found)
-            return CM_ERROR_NOSUCHFILE;
-
-        unlink(hfile);
-        rename(hfile2, hfile);
+        if (found) {
+            unlink(hfile);
+            rename(hfile2, hfile);
+        }
     }
     
     lock_ReleaseMutex(&cm_Freelance_Lock);
-    cm_noteLocalMountPointChange();
-    return 0;
+    if (found) {
+        cm_noteLocalMountPointChange();
+        return 0;
+    } else
+        return CM_ERROR_NOSUCHFILE;
 }
 
 long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
@@ -1246,8 +1245,8 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
     }
 
 #if !defined(DJGPP)
-    if ( cm_FreelanceMountPointExists(filename) ||
-         cm_FreelanceSymlinkExists(filename) )
+    if ( cm_FreelanceMountPointExists(filename, 0) ||
+         cm_FreelanceSymlinkExists(filename, 0) )
         return CM_ERROR_EXISTS;
 #endif
 
@@ -1312,10 +1311,8 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
     /* cm_reInitLocalMountPoints(); */
-    if (fidp) {
-        fidp->unique = 1;
-        fidp->vnode = cm_noLocalMountPoints + 1;   /* vnode value of last mt pt */
-    }
+    if (fidp)
+        cm_SetFid(fidp, fidp->cell, fidp->volume, ++cm_noLocalMountPoints, 1);
     cm_noteLocalMountPointChange();
     return 0;
 }
@@ -1369,6 +1366,7 @@ long cm_FreelanceRemoveSymlink(char *toremove)
 
             if (!strcmp(shortname, toremove)) {
                 RegDeleteValue( hkFreelanceSymlinks, szValueName );
+                found = 1;
                 break;
             }
         }
@@ -1377,7 +1375,10 @@ long cm_FreelanceRemoveSymlink(char *toremove)
 #endif
     
     lock_ReleaseMutex(&cm_Freelance_Lock);
-    cm_noteLocalMountPointChange();
-    return 0;
+    if (found) {
+        cm_noteLocalMountPointChange();
+        return 0;
+    } else
+        return CM_ERROR_NOSUCHFILE;
 }
 #endif /* AFS_FREELANCE_CLIENT */
