@@ -223,10 +223,10 @@ typedef struct myncb {
 typedef struct smb_vc {
     struct smb_vc *nextp;		/* not used */
     afs_uint32 magic;			/* a magic value to detect bad entries */
-    unsigned long refCount;		/* the reference count */
-    long flags;			        /* the flags, if any; locked by mx */
+    afs_int32 refCount;		        /* the reference count */
+    afs_uint32 flags;		        /* the flags, if any; locked by mx */
     osi_mutex_t mx;			/* the mutex */
-    long vcID;			        /* VC id */
+    afs_uint32 vcID;		        /* VC id */
     unsigned short lsn;		        /* the NCB LSN associated with this */
     unsigned short uidCounter;	        /* session ID counter */
     unsigned short tidCounter;	        /* tree ID counter */
@@ -259,7 +259,7 @@ typedef struct smb_vc {
 /* one per user session */
 typedef struct smb_user {
     struct smb_user *nextp;		/* next sibling */
-    unsigned long refCount;		/* ref count */
+    afs_int32 refCount;		        /* ref count */
     afs_uint32 flags;		        /* flags; locked by mx */
     osi_mutex_t mx;
     unsigned short userID;		/* the session identifier */
@@ -272,7 +272,7 @@ typedef struct smb_user {
 
 typedef struct smb_username {
     struct smb_username *nextp;		/* next sibling */
-    unsigned long refCount;		/* ref count */
+    afs_int32 refCount;		        /* ref count */
     long flags;			        /* flags; locked by mx */
     osi_mutex_t mx;
     struct cm_user *userp;		/* CM user structure */
@@ -303,7 +303,7 @@ typedef struct smb_username {
 /* one per tree-connect */
 typedef struct smb_tid {
     struct smb_tid *nextp;		/* next sibling */
-    unsigned long refCount;
+    afs_int32 refCount;
     afs_uint32 flags;			/* protected by mx */
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short tid;		        /* the tid */
@@ -319,7 +319,7 @@ typedef struct smb_tid {
 /* one per process ID */
 typedef struct smb_pid {
     struct smb_pid *nextp;		/* next sibling */
-    unsigned long refCount;
+    afs_int32 refCount;
     long flags;
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short pid;		        /* the pid */
@@ -360,7 +360,7 @@ typedef struct smb_ioctl {
 /* one per file ID; these are really file descriptors */
 typedef struct smb_fid {
     osi_queue_t q;
-    unsigned long refCount;
+    afs_int32 refCount;
     afs_uint32 flags;			/* protected by mx */
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short fid;		        /* the file ID */
@@ -452,7 +452,7 @@ typedef struct smb_fid {
 typedef struct smb_dirSearch {
     osi_queue_t q;			/* queue of all outstanding cookies */
     osi_mutex_t mx;			/* just in case the caller screws up */
-    unsigned long refCount;		/* reference count */
+    afs_int32 refCount;		        /* reference count */
     long cookie;			/* value returned to the caller */
     struct cm_scache *scp;		/* vnode of the dir we're searching */
     time_t lastTime;			/* last time we used this (osi_Time) */
@@ -495,9 +495,10 @@ typedef struct smb_waitingLock {
     int              state;
 } smb_waitingLock_t;
 
-#define SMB_WAITINGLOCKSTATE_WAITING 0
-#define SMB_WAITINGLOCKSTATE_DONE    1
-#define SMB_WAITINGLOCKSTATE_ERROR   2
+#define SMB_WAITINGLOCKSTATE_WAITING   0
+#define SMB_WAITINGLOCKSTATE_DONE      1
+#define SMB_WAITINGLOCKSTATE_ERROR     2
+#define SMB_WAITINGLOCKSTATE_CANCELLED 3
 
 /* waiting lock request */
 typedef struct smb_waitingLockRequest {
@@ -507,7 +508,8 @@ typedef struct smb_waitingLockRequest {
     smb_packet_t *inp;
     smb_packet_t *outp;
     int           lockType;
-    time_t        timeRemaining;
+    time_t        start_t;              /* osi_Time */
+    afs_uint32    msTimeout;            /* msecs, 0xFFFFFFFF = wait forever */
     smb_waitingLock_t * locks;
 } smb_waitingLockRequest_t;
 
@@ -563,7 +565,7 @@ extern smb_tid_t *smb_FindTID(smb_vc_t *vcp, unsigned short tid, int flags);
 
 extern void smb_HoldTIDNoLock(smb_tid_t *tidp);
 
-extern void smb_ReleaseTID(smb_tid_t *tidp);
+extern void smb_ReleaseTID(smb_tid_t *tidp, afs_uint32 locked);
 
 extern smb_user_t *smb_FindUID(smb_vc_t *vcp, unsigned short uid, int flags);
 
@@ -671,6 +673,8 @@ extern int smb_maxMpxRequests; /* max # of mpx requests */
 extern int smb_StoreAnsiFilenames;
 extern int smb_hideDotFiles;
 extern unsigned int smb_IsDotFile(char *lastComp);
+extern afs_uint32 smb_AsyncStore;
+extern afs_uint32 smb_AsyncStoreSize;
 
 /* the following are used for smb auth */
 extern int smb_authType; /* Type of SMB authentication to be used. One from below. */
@@ -727,18 +731,18 @@ extern unsigned char *smb_ParseVblBlock(unsigned char *inp, char **chainpp, int
 extern int smb_SUser(cm_user_t *userp);
 
 #ifndef DJGPP
-long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count, char *op,
+long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, afs_uint32 count, char *op,
 	cm_user_t *userp, long *writtenp);
 #else /* DJGPP */
-long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count, char *op,
+long smb_WriteData(smb_fid_t *fidp, osi_hyper_t *offsetp, afs_uint32 count, char *op,
 	cm_user_t *userp, long *writtenp, int dosflag);
 #endif /* !DJGPP */
 
 #ifndef DJGPP
-extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count,
+extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, afs_uint32 count,
 	char *op, cm_user_t *userp, long *readp);
 #else /* DJGPP */
-extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, long count,
+extern long smb_ReadData(smb_fid_t *fidp, osi_hyper_t *offsetp, afs_uint32 count,
 	char *op, cm_user_t *userp, long *readp, int dosflag);
 #endif /* !DJGPP */
 
@@ -755,11 +759,13 @@ extern DWORD smb_ServerExceptionFilter(void);
 extern void smb_UpdateServerPriority(void);
 extern void smb_SetRequestStartTime(void);
 extern void smb_ResetServerPriority(void);
-extern void smb_RestartListeners(void);
-extern void smb_StopListeners(void);
-extern void smb_StopListener(NCB *ncbp, int lana);
+extern void smb_RestartListeners(int);
+extern void smb_StopListeners(int);
+extern void smb_StopListener(NCB *ncbp, int lana, int wait);
 extern long smb_IsNetworkStarted(void);
-extern void smb_LanAdapterChange(void);
+extern void smb_LanAdapterChange(int);
+extern void smb_SetLanAdapterChangeDetected(void);
+
 
 #define SMB_LISTENER_UNINITIALIZED -1
 #define SMB_LISTENER_STOPPED 0
