@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005,2006 Secure Endpoints Inc.
+ * Copyright (c) 2005,2006,2007, 2008 Secure Endpoints Inc.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -22,7 +22,7 @@
  * SOFTWARE.
  */
 
-/* $Id: afsfuncs.c,v 1.1.2.12 2007/08/29 05:32:34 jaltman Exp $ */
+/* $Id: afsfuncs.c,v 1.1.2.18 2008/03/30 03:36:57 jaltman Exp $ */
 
 /* Disable the 'macro redefinition' warning which is getting
    triggerred by a redefinition of the ENCRYPT and DECRYPT macros. */
@@ -703,6 +703,24 @@ ViceIDToUsername(char *username,
 }
 
 
+static void
+copy_realm_of_ticket(krb5_context context, char * dest, size_t destlen, krb5_creds *v5cred) {
+    krb5_error_code code;
+    krb5_ticket *ticket;
+    size_t len;
+
+    code = pkrb5_decode_ticket(&v5cred->ticket, &ticket);
+    if (code == 0) {
+        len = krb5_princ_realm(context, ticket->server)->length;
+        if (len > destlen - 1)
+            len = destlen - 1;
+
+        StringCbCopyA(dest, len, krb5_princ_realm(context, ticket->server)->data);
+
+        pkrb5_free_ticket(context, ticket);
+    }
+}
+
 int
 afs_klog(khm_handle identity,
          char *service,
@@ -727,7 +745,7 @@ afs_klog(khm_handle identity,
     char	CellName[128];
     char	ServiceName[128];
     khm_handle	confighandle = NULL;
-    khm_int32	supports_krb4 = 1;
+    khm_int32	supports_krb4 = (pkrb_get_tf_realm == NULL ? 0 : 1);
     khm_int32   got524cred = 0;
 
     /* signalling */
@@ -823,17 +841,6 @@ afs_klog(khm_handle identity,
             goto try_krb4;
         }
 
-        /* First try Service/Cell@REALM */
-        if (r = pkrb5_build_principal(context, &increds.server,
-                                         (int) strlen(RealmName),
-                                         RealmName,
-                                         ServiceName,
-                                         CellName,
-                                         0)) {
-            _reportf(L"krb5_build_principal returns %d", r);
-            goto end_krb5;
-        }
-
         increds.client = client_principal;
         increds.times.endtime = 0;
         /* Ask for DES since that is what V4 understands */
@@ -843,50 +850,148 @@ afs_klog(khm_handle identity,
         flags = KRB5_TC_OPENCLOSE;
         r = pkrb5_cc_set_flags(context, k5cc, flags);
 #endif
-      retry_retcred:
-        r = pkrb5_get_credentials(context, 0, k5cc, &increds, &k5creds);
-	if ((r == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN ||
-	      r == KRB5KRB_ERR_GENERIC /* Heimdal */) &&
-	     !RealmName[0]) {
-	    StringCbCopyA(RealmName, sizeof(RealmName), 
-			  afs_realm_of_cell(&ak_cellconfig, TRUE));
+        if (strlen(realm) != 0) {
+          retry_retcred_1:
+            /* First try Service/Cell@REALM */
+            if (r = pkrb5_build_principal(context, &increds.server,
+                                           (int) strlen(realm),
+                                           realm,
+                                           ServiceName,
+                                           CellName,
+                                           0)) {
+                _reportf(L"krb5_build_principal returns %d", r);
+                goto end_krb5;
+            }
 
-            pkrb5_free_principal(context, increds.server);
-	    r = pkrb5_build_principal(context, &increds.server,
-					 (int) strlen(RealmName),
-					 RealmName,
-					 ServiceName,
-					 CellName,
-					 0);
-            if (r == 0)
-                r = pkrb5_get_credentials(context, 0, k5cc, 
-                                          &increds, &k5creds);
-	}
-	if (r == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN ||
-            r == KRB5KRB_ERR_GENERIC /* Heimdal */) {
-            /* Next try Service@REALM */
-            pkrb5_free_principal(context, increds.server);
-            r = pkrb5_build_principal(context, &increds.server,
-                                         (int) strlen(RealmName),
-                                         RealmName,
-                                         ServiceName,
-                                         0);
-            if (r == 0)
-                r = pkrb5_get_credentials(context, 0, k5cc, 
-                                          &increds, &k5creds);
+            r = pkrb5_get_credentials(context, 0, k5cc, &increds, &k5creds);
+            if (r == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN ||
+                r == KRB5_ERR_HOST_REALM_UNKNOWN ||
+                r == KRB5KRB_ERR_GENERIC /* Heimdal */) {
+                /* Next try Service@REALM */
+                pkrb5_free_principal(context, increds.server);
+                r = pkrb5_build_principal(context, &increds.server,
+                                           (int) strlen(realm),
+                                           realm,
+                                           ServiceName,
+                                           0);
+                if (r == 0)
+                    r = pkrb5_get_credentials(context, 0, k5cc, 
+                                               &increds, &k5creds);
+            }
+
+            /* Check to make sure we received a valid ticket; if not remove it
+             * and try again.  Perhaps there are two service tickets for the
+             * same service in the ccache.
+             */
+            if (r == 0 && k5creds && k5creds->times.endtime < time(NULL)) {
+                pkrb5_free_principal(context, increds.server);
+                pkrb5_cc_remove_cred(context, k5cc, 0, k5creds);
+                pkrb5_free_creds(context, k5creds);
+                k5creds = NULL;
+                goto retry_retcred_1;
+            }
+        } else {
+          retry_retcred_2:
+            /* First try Service/Cell@_CLIENT_REALM */
+            if (r = pkrb5_build_principal(context, &increds.server,
+                                           (int) strlen(realm_of_user),
+                                           realm_of_user,
+                                           ServiceName,
+                                           CellName,
+                                           0)) {
+                _reportf(L"krb5_build_principal returns %d", r);
+                goto end_krb5;
+            }
+
+            r = pkrb5_get_credentials(context, 0, k5cc, &increds, &k5creds);
+            if (r == 0) {
+                /* the user realm is a valid cell realm */
+                StringCbCopyA(realm_of_cell, sizeof(realm_of_cell), realm_of_user);
+            }
+            if (r == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN ||
+                r == KRB5_ERR_HOST_REALM_UNKNOWN ||
+                r == KRB5KRB_ERR_GENERIC /* Heimdal */) {
+                pkrb5_free_principal(context, increds.server);
+                r = pkrb5_build_principal(context, &increds.server,
+                                           (int) strlen(realm_of_cell),
+                                           realm_of_cell,
+                                           ServiceName,
+                                           CellName,
+                                           0);
+                if (r == 0)
+                    r = pkrb5_get_credentials(context, 0, k5cc, 
+                                               &increds, &k5creds);
+            }
+            if ((r == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN ||
+                 r == KRB5_ERR_HOST_REALM_UNKNOWN ||
+                 r == KRB5KRB_ERR_GENERIC /* Heimdal */) &&
+                 strlen(realm_of_cell) == 0) {
+                StringCbCopyA(realm_of_cell, sizeof(realm_of_cell), 
+                               afs_realm_of_cell(&ak_cellconfig, TRUE));
+
+                pkrb5_free_principal(context, increds.server);
+                r = pkrb5_build_principal(context, &increds.server,
+                                           (int) strlen(realm_of_cell),
+                                           realm_of_cell,
+                                           ServiceName,
+                                           CellName,
+                                           0);
+                if (r == 0)
+                    r = pkrb5_get_credentials(context, 0, k5cc, 
+                                               &increds, &k5creds);
+            }
+            if (r == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN ||
+                r == KRB5_ERR_HOST_REALM_UNKNOWN ||
+                r == KRB5KRB_ERR_GENERIC /* Heimdal */) {
+                /* Next try Service@REALM */
+                StringCbCopyA(realm_of_cell, sizeof(realm_of_cell), 
+                               afs_realm_of_cell(&ak_cellconfig, FALSE));
+
+                pkrb5_free_principal(context, increds.server);
+                r = pkrb5_build_principal(context, &increds.server,
+                                           (int) strlen(realm_of_cell),
+                                           realm_of_cell,
+                                           ServiceName,
+                                           0);
+                if (r == 0)
+                    r = pkrb5_get_credentials(context, 0, k5cc, 
+                                               &increds, &k5creds);
+            }
+            if ((r == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN ||
+                 r == KRB5_ERR_HOST_REALM_UNKNOWN ||
+                 r == KRB5KRB_ERR_GENERIC /* Heimdal */) &&
+                 strlen(realm_of_cell) == 0) {
+                /* Next try Service@REALM */
+                StringCbCopyA(realm_of_cell, sizeof(realm_of_cell), 
+                               afs_realm_of_cell(&ak_cellconfig, TRUE));
+
+                pkrb5_free_principal(context, increds.server);
+                r = pkrb5_build_principal(context, &increds.server,
+                                           (int) strlen(realm_of_cell),
+                                           realm_of_cell,
+                                           ServiceName,
+                                           0);
+                if (r == 0)
+                    r = pkrb5_get_credentials(context, 0, k5cc, 
+                                               &increds, &k5creds);
+            }
+
+            if (r == 0 && strlen(realm_of_cell) == 0) 
+                copy_realm_of_ticket(context, realm_of_cell, sizeof(realm_of_cell), k5creds);
+
+            /* Check to make sure we received a valid ticket; if not remove it
+             * and try again.  Perhaps there are two service tickets for the
+             * same service in the ccache.
+             */
+            if (r == 0 && k5creds && k5creds->times.endtime < time(NULL)) {
+                pkrb5_free_principal(context, increds.server);
+                pkrb5_cc_remove_cred(context, k5cc, 0, k5creds);
+                pkrb5_free_creds(context, k5creds);
+                k5creds = NULL;
+                goto retry_retcred_2;
+            }
         }
 
-	/* Check to make sure we received a valid ticket; if not remove it
-	* and try again.  Perhaps there are two service tickets for the
-	* same service in the ccache.
-	*/
-	if (r == 0 && k5creds && k5creds->times.endtime < time(NULL)) {
-	    pkrb5_cc_remove_cred(context, k5cc, 0, k5creds);
-	    pkrb5_free_creds(context, k5creds);
-	    k5creds = NULL;
-	    goto retry_retcred;
-	}
-
         pkrb5_free_principal(context, increds.server);
         pkrb5_free_principal(context, client_principal);
         client_principal = 0;
@@ -900,6 +1005,7 @@ afs_klog(khm_handle identity,
 
         if (r) {
             _reportf(L"Code %d while getting credentials", r);
+            k5creds = NULL;
             goto end_krb5;
         }
 
@@ -1008,8 +1114,8 @@ afs_klog(khm_handle identity,
 
         _reportf(L"Trying Krb524");
 
-        if (method == AFS_TOKEN_AUTO ||
-            method == AFS_TOKEN_KRB524) {
+        if (pkrb524_convert_creds_kdc && 
+            (method == AFS_TOKEN_AUTO || method == AFS_TOKEN_KRB524)) {
             /* This requires krb524d to be running with the KDC */
             r = pkrb524_convert_creds_kdc(context, k5creds, &creds);
             if (r) {
@@ -1035,14 +1141,15 @@ afs_klog(khm_handle identity,
     /* Kerberos 4 */
  try_krb4:
 
-    kcdb_identity_get_config(identity, 0, &confighandle);
-    khc_read_int32(confighandle, L"Krb4Cred\\Krb4NewCreds", &supports_krb4);
-    khc_close_space(confighandle);
-
-    if (!supports_krb4) {
-        _reportf(L"Kerberos 4 not configured");
+    if (supports_krb4) {
+	kcdb_identity_get_config(identity, 0, &confighandle);
+	khc_read_int32(confighandle, L"Krb4Cred\\Krb4NewCreds", &supports_krb4);
+	khc_close_space(confighandle); 
     }
 
+    if (!supports_krb4)
+        _reportf(L"Kerberos 4 not configured");
+
     if (!bGotCreds && supports_krb4 && 
         (method == AFS_TOKEN_AUTO ||
          method == AFS_TOKEN_KRB4)) {
