@@ -499,6 +499,7 @@ extern void shutdown_nfsclnt(void);
 /* afs_osi.c */
 extern afs_lock_t afs_ftf;
 extern void afs_osi_Invisible(void);
+extern void afs_osi_Visible(void);
 extern void afs_osi_RxkRegister(void);
 extern void afs_osi_MaskSignals(void);
 extern void afs_osi_UnmaskRxkSignals(void);
@@ -591,7 +592,6 @@ extern void osi_linux_free_inode_pages(void);
 #endif
 extern void osi_linux_mask(void);
 extern void osi_linux_unmask(void);
-extern void osi_linux_rxkreg(void);
 extern int setpag(cred_t ** cr, afs_uint32 pagvalue, afs_uint32 * newpag,
 		  int change_parent);
 #endif
@@ -746,7 +746,7 @@ extern struct server *afs_GetServer(afs_uint32 * aserver, afs_int32 nservers,
 				    afs_int32 addr_uniquifier);
 extern void ForceAllNewConnections(void);
 extern void afs_MarkServerUpOrDown(struct srvAddr *sa, int a_isDown);
-extern void afs_ServerDown(struct srvAddr *sa);
+extern afs_int32 afs_ServerDown(struct srvAddr *sa);
 extern void afs_CountServers(void);
 extern void afs_CheckServers(int adown, struct cell *acellp);
 extern unsigned int afs_random(void);
@@ -850,6 +850,9 @@ extern char *afs_strcpy(char *s1, char *s2);
 #ifndef afs_strchr
 extern char *afs_strchr(char *s, int c);
 #endif
+#ifndef afs_strrchr
+extern char *afs_strrchr(char *s, int c);
+#endif
 extern char *afs_strdup(char *s);
 extern void print_internet_address(char *preamble, struct srvAddr *sa,
 				   char *postamble, int flag);
