@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bozo/bnode.c,v 1.19.2.2 2006/11/09 02:16:23 shadow Exp $");
+    ("$Header: /cvs/openafs/src/bozo/bnode.c,v 1.19.2.6 2008/03/10 22:32:32 shadow Exp $");
 
 #include <stddef.h>
 #include <stdlib.h>
@@ -28,14 +28,7 @@ RCSID
 #include <time.h>
 #endif
 #include <sys/stat.h>
-
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <lwp.h>
@@ -43,6 +36,7 @@ RCSID
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
 #include "bnode.h"
+#include "bosprototypes.h"
 
 #if defined(AFS_AIX_ENV) || defined(AFS_SUN4_ENV)
 /* All known versions of AIX lack WCOREDUMP but this works */
@@ -317,7 +311,7 @@ bnode_Register(char *atype, struct bnode_ops *aprocs, int anparms)
 afs_int32
 bnode_Create(char *atype, char *ainstance, struct bnode ** abp, char *ap1,
 	     char *ap2, char *ap3, char *ap4, char *ap5, char *notifier,
-	     int fileGoal)
+	     int fileGoal, int rewritefile)
 {
     struct bnode_type *type;
     struct bnode *tb;
@@ -360,7 +354,10 @@ bnode_Create(char *atype, char *ainstance, struct bnode ** abp, char *ap1,
     tb->fileGoal = fileGoal;
 
     bnode_SetStat(tb, tb->goal);	/* nudge it once */
-    WriteBozoFile(0);
+
+    if (rewritefile != 0)
+	WriteBozoFile(0);
+
     return 0;
 }
 
@@ -494,8 +491,8 @@ DeleteProc(register struct bnode_proc *abproc)
 }
 
 /* bnode lwp executes this code repeatedly */
-static int
-bproc()
+static void *
+bproc(void *unused)
 {
     register afs_int32 code;
     register struct bnode *tb;
@@ -644,6 +641,7 @@ bproc()
 	    }
 	}
     }
+    return NULL;
 }
 
 static afs_int32
@@ -740,7 +738,7 @@ hdl_notifier(struct bnode_proc *tp)
 	ec = setsid();
 #elif defined(AFS_DARWIN90_ENV)
 	ec = setpgid(0, 0);
-#elif defined(AFS_LINUX20_ENV) || defined(AFS_AIX_ENV) 
+#elif defined(AFS_LINUX20_ENV) || defined(AFS_AIX_ENV)  
 	ec = setpgrp();
 #else
 	ec = setpgrp(0, 0);
@@ -766,9 +764,11 @@ hdl_notifier(struct bnode_proc *tp)
 
 /* Called by IOMGR at low priority on IOMGR's stack shortly after a SIGCHLD
  * occurs.  Wakes up bproc do redo things */
-int
-bnode_SoftInt(int asignal)
+void *
+bnode_SoftInt(void *param)
 {
+    /* int asignal = (int) param; */
+
     IOMGR_Cancel(bproc_pid);
     return 0;
 }
@@ -779,12 +779,10 @@ bnode_SoftInt(int asignal)
 void
 bnode_Int(int asignal)
 {
-    extern void bozo_ShutdownAndExit();
-
     if (asignal == SIGQUIT) {
-	IOMGR_SoftSig(bozo_ShutdownAndExit, (char *)asignal);
+	IOMGR_SoftSig(bozo_ShutdownAndExit, (void *) asignal);
     } else {
-	IOMGR_SoftSig(bnode_SoftInt, (char *)asignal);
+	IOMGR_SoftSig(bnode_SoftInt, (void *) asignal);
     }
 }
 
