@@ -75,6 +75,12 @@
 #endif
 #endif /* KERNEL */
 
+#ifdef __GLIBC__
+#if (__GLIBC__ > 2) || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 3)
+#define USE_UCONTEXT
+#endif
+#endif
+
 #endif /* _PARAM_PPC64_LINUX20_H_ */
 
 #else /* !defined(UKERNEL) */
