@@ -12,7 +12,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/kpasswd.c,v 1.15.8.1 2007/04/10 18:39:52 shadow Exp $");
+    ("$Header: /cvs/openafs/src/kauth/kpasswd.c,v 1.15.8.3 2007/10/31 04:09:30 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -29,13 +29,7 @@ RCSID
 #ifndef AFS_NT40_ENV
 #include <pwd.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <signal.h>
 #include <afs/com_err.h>
 #include <afs/auth.h>
@@ -79,7 +73,7 @@ RCSID
 
 /* The following code to make use of libcmd.a also stolen from klog.c. */
 
-int CommandProc();
+int CommandProc(struct cmd_syndesc *, void *);
 
 static int zero_argc;
 static char **zero_argv;
@@ -207,9 +201,7 @@ timedout()
 #endif
 
 char passwd[BUFSIZ], npasswd[BUFSIZ], verify[BUFSIZ];
-CommandProc(as, arock)
-     char *arock;
-     struct cmd_syndesc *as;
+CommandProc(struct cmd_syndesc *as, void *arock)
 {
     char name[MAXKTCNAMELEN] = "";
     char instance[MAXKTCNAMELEN] = "";
