@@ -15,7 +15,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_rdwr.c,v 1.29.2.1 2006/06/01 15:50:21 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_rdwr.c,v 1.29.2.4 2008/05/08 21:25:58 shadow Exp $");
 
 #ifdef KERNEL
 #ifndef UKERNEL
@@ -81,13 +81,7 @@ RCSID
 # include <sys/stat.h>
 # include <sys/time.h>
 #endif /* !AFS_NT40_ENV */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -198,8 +192,9 @@ rxi_ReadProc(register struct rx_call *call, register char *buf,
 					       RX_CALL_REFCOUNT_DELAY);
 				rxi_SendAck(call, 0, 0, RX_ACK_DELAY, 0);
 			    } else {
-				struct clock when;
-				clock_GetTime(&when);
+				struct clock when, now;
+				clock_GetTime(&now);
+				when = now;
 				/* Delay to consolidate ack packets */
 				clock_Add(&when, &rx_hardAckDelay);
 				if (!call->delayedAckEvent
@@ -210,7 +205,7 @@ rxi_ReadProc(register struct rx_call *call, register char *buf,
 						   RX_CALL_REFCOUNT_DELAY);
 				    CALL_HOLD(call, RX_CALL_REFCOUNT_DELAY);
 				    call->delayedAckEvent =
-					rxevent_Post(&when,
+				      rxevent_PostNow(&when, &now,
 						     rxi_SendDelayedAck, call,
 						     0);
 				}
@@ -527,8 +522,9 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 	    rxi_SendAck(call, 0, serial, RX_ACK_DELAY, 0);
 	    didHardAck = 1;
 	} else {
-	    struct clock when;
-	    clock_GetTime(&when);
+	    struct clock when, now;
+	    clock_GetTime(&now);
+	    when = now;
 	    /* Delay to consolidate ack packets */
 	    clock_Add(&when, &rx_hardAckDelay);
 	    if (!call->delayedAckEvent
@@ -537,7 +533,7 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 			       RX_CALL_REFCOUNT_DELAY);
 		CALL_HOLD(call, RX_CALL_REFCOUNT_DELAY);
 		call->delayedAckEvent =
-		    rxevent_Post(&when, rxi_SendDelayedAck, call, 0);
+		    rxevent_PostNow(&when, &now, rxi_SendDelayedAck, call, 0);
 	    }
 	}
     }
@@ -704,7 +700,7 @@ rxi_WriteProc(register struct rx_call *call, register char *buf,
 	    }
 	    /* Wait for transmit window to open up */
 	    while (!call->error
-		   && call->tnext + 1 > call->tfirst + call->twind) {
+		   && call->tnext + 1 > call->tfirst + (2 * call->twind)) {
 		clock_NewTime();
 		call->startWait = clock_Sec();
 
@@ -1142,7 +1138,7 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
     }
 
     /* Wait for the length of the transmit queue to fall below call->twind */
-    while (!call->error && call->tnext + 1 > call->tfirst + call->twind) {
+    while (!call->error && call->tnext + 1 > call->tfirst + (2 * call->twind)) {
 	clock_NewTime();
 	call->startWait = clock_Sec();
 #ifdef	RX_ENABLE_LOCKS
