@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/phys.c,v 1.9.2.2 2007/12/10 22:45:55 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/phys.c,v 1.9.2.4 2008/04/28 21:48:11 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -23,7 +23,14 @@ RCSID
 #include <netinet/in.h>
 #endif
 #include <sys/stat.h>
+
+/* #if defined (AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV) */
+#if 0   /* temporary hack - klm */
+/* nothing */
+#else
 #include <lwp.h>
+#endif
+
 #include <lock.h>
 #include <errno.h>
 #include <string.h>
@@ -130,7 +137,7 @@ uphys_close(register int afd)
 	return EBADF;
     tfd = fdcache;
     for (i = 0; i < MAXFDCACHE; i++, tfd++) {
-	if (tfd->fd == afd) {
+	if (tfd->fd == afd && tfd->fileID != -10000) {
 	    tfd->refCount--;
 	    return 0;
 	}
@@ -282,3 +289,20 @@ uphys_sync(register struct ubik_dbase *adbase, afs_int32 afile)
     uphys_close(fd);
     return code;
 }
+
+void
+uphys_invalidate(register struct ubik_dbase *adbase, afs_int32 afid)
+{
+    register int i;
+    register struct fdcache *tfd;
+
+    /* scan file descr cache */
+    for (tfd = fdcache, i = 0; i < MAXFDCACHE; i++, tfd++) {
+	if (afid == tfd->fileID) {
+	    tfd->fileID = -10000;
+	    if (tfd->fd >= 0 && tfd->refCount == 0)
+		close(tfd->fd);
+	    return;
+	}
+    }
+}
