@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/vote.c,v 1.13 2003/07/15 23:17:06 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/vote.c,v 1.14.14.1 2007/10/30 15:16:47 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -22,13 +22,7 @@ RCSID
 #endif
 #include <afs/afsutil.h>
 #include <lock.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <afs/afsutil.h>
@@ -546,6 +540,7 @@ ubik_dprint(char *a, char *b, char *c, char *d, char *e, char *f, char *g,
 	    char *h)
 {
     ViceLog(5, (a, b, c, d, e, f, g, h));
+    return 0;
 }
 
 int
@@ -553,6 +548,7 @@ ubik_print(char *a, char *b, char *c, char *d, char *e, char *f, char *g,
 	   char *h)
 {
     ViceLog(0, (a, b, c, d, e, f, g, h));
+    return 0;
 }
 
 /* called once/run to init the vote module */
